@@ -18,6 +18,23 @@
 --      Boston, MA 02111-1307, USA.
 --
 -- $Log: prep_disk.c,v $
+-- Revision 1.39  2007/06/28 14:09:15  jsedwards
+-- Fix seg fault if no parameters were given on the command line.
+--
+-- Revision 1.38  2007/06/26 20:04:18  jsedwards
+-- Add call to allocate the chunk for the root object.
+--
+-- Revision 1.37  2007/06/26 11:55:05  jsedwards
+-- Changes for new non-wrapped, chunk indexed format (0023).
+--
+-- Revision 1.36  2007/06/25 15:07:11  jsedwards
+-- Move call to agree with warranty after parameters are checked, so that it
+-- doesn't ask about the warranty if the parameters are bad.
+--
+-- Revision 1.35  2007/06/25 15:02:15  jsedwards
+-- Fix bug in convert_size so it correctly returns SIZE_ERROR when there is an
+-- error.
+--
 -- Revision 1.34  2007/06/23 14:38:37  jsedwards
 -- Removed #ifndef DONT_COPY_PUBLIC_BLOCKS statements that are no longer
 -- necessary since we don't copy the public blocks into the private storage
@@ -252,7 +269,7 @@ void print_usage(char* name, bool specify_private_size)
     }
     else
     {
-	fprintf(stderr, "usage: %s [--no-private] private_blocks\n\n", name);
+	fprintf(stderr, "usage: %s [--no-private] size\n\n", name);
     }
 
     fprintf(stderr, "  --no-private: don't create the initial private blocks (if importing old data)\n");
@@ -387,19 +404,25 @@ uint32 convert_size(char* size)
     if (!isdigit(*size) || *p != 0)
     {
 	fprintf(stderr, "Invalid size: %s\n", size);
-	result = SIZE_ERROR;
+	return SIZE_ERROR;
     }
 
     if (result == 0 || (result % (CHUNK_SIZE / 1048576)) != 0)
     {
 	fprintf(stderr, "Size must be a multiple of: %dM\n", CHUNK_SIZE / 1048576);
-	result = SIZE_ERROR;
+	return SIZE_ERROR;
+    }
+
+    if (result < 2 * CHUNK_SIZE / 1048576)
+    {
+	fprintf(stderr, "Size must be equal to, or larger than %dM\n", 2 * CHUNK_SIZE / 1048576);
+	return SIZE_ERROR;
     }
 
     if (result > 1048576)
     {
 	fprintf(stderr, "Size must be less than a 1024 gigabytes\n");
-	result = SIZE_ERROR;
+	return SIZE_ERROR;
     }
 
     return result * (1048576 / FILE_BLOCK_SIZE);   /* convert megabytes to blocks */
@@ -482,11 +505,11 @@ int main(int argc, char* argv[])
     bool is_file = false;
     bool do_private_blocks = true;
 
+    uint32 total_blocks;
     uint32 reserved_private_blocks;
     uint32 blocks = 0;
     uint32 blocks_written;
     uint32 num_chunks;
-    float chunks;
     Disk_Header disk_header;
     ObjRef root_obj_ref;
     ObjRef private_root_obj_ref;
@@ -503,8 +526,6 @@ int main(int argc, char* argv[])
 //    struct stat stat_struct;
 #endif
 
-    agree_no_warranty();   /* make sure he or she agrees with the licensing */
-
     printf("\n");
 
     nwos_log_arguments(argc, argv);
@@ -519,7 +540,7 @@ int main(int argc, char* argv[])
 	is_file = true;
     }
 
-    if (*argv[arg_index] == '-')   /* option specified */
+    if (argc > 1 && *argv[arg_index] == '-')   /* option specified */
     {
 	if (strcmp(argv[arg_index], "--no-private") != 0)
 	{
@@ -559,25 +580,32 @@ int main(int argc, char* argv[])
 
     if (!is_file)
     {
-	reserved_private_blocks = max_blocks_on_device(DEFAULT_FILE);   /* zero means unlimited */
+	total_blocks = max_blocks_on_device(DEFAULT_FILE);   /* zero means unlimited */
 
-	num_chunks = reserved_private_blocks / BLOCKS_IN_CHUNK;
+	num_chunks = total_blocks / BLOCKS_IN_CHUNK;
 
-	/* leave one chunk for header and index */
-	reserved_private_blocks = (num_chunks - 1) * BLOCKS_IN_CHUNK;
+	/* round it to an even number of chunks */
+	total_blocks = num_chunks * BLOCKS_IN_CHUNK;
     }
     else
     {
-	reserved_private_blocks = convert_size(argv[arg_index]);
+	/* use one chunk for header and block map */
+	total_blocks = convert_size(argv[arg_index]);
 
-	if (reserved_private_blocks == 0 || reserved_private_blocks == SIZE_ERROR)
+	if (total_blocks == 0 || total_blocks == SIZE_ERROR)
 	{
 	    print_usage(argv[0], is_file);
 	    exit(1);
 	}
     }
 
-    printf("\nreserved private chunks: %u\n", reserved_private_blocks / BLOCKS_IN_CHUNK);
+
+    agree_no_warranty();   /* make sure he or she agrees with the licensing */
+
+    reserved_private_blocks = total_blocks - BLOCKS_IN_CHUNK;  /* first chunk is for header */
+
+    printf("\ntotal chunks: %u  private chunks: %u\n",
+	   total_blocks / BLOCKS_IN_CHUNK, reserved_private_blocks / BLOCKS_IN_CHUNK);
 
 
     chunk = malloc(CHUNK_SIZE);
@@ -644,11 +672,18 @@ int main(int argc, char* argv[])
 
     /* compute the number of private blocks used */
 
-    num_chunks = (reserved_private_blocks + BLOCKS_IN_CHUNK - 1) / BLOCKS_IN_CHUNK;  /* round up */
+    assert(reserved_private_blocks % BLOCKS_IN_CHUNK == 0);
 
-    blocks = num_chunks * BIT_MAP_BLOCKS;
+    num_chunks = reserved_private_blocks / BLOCKS_IN_CHUNK;  /* round up */
+
+    /* set the number of used blocks to 0 */
+    blocks = 0;
     nwos_uint32_to_4_uint8(&blocks, disk_header.used_blocks);
 
+    /* set the number of used chunks to 0 */
+    blocks = 0;
+    nwos_uint32_to_4_uint8(&blocks, disk_header.used_chunks);
+
     /* for now set the chunk offset to 1 chunk (for the header) */
     blocks = BLOCKS_IN_CHUNK;
     nwos_uint32_to_4_uint8(&blocks, disk_header.block_offset_to_chunks);
@@ -669,40 +704,44 @@ int main(int argc, char* argv[])
     /* write private portion of the disk */
     /*************************************/
 
-    printf("Writing private chunks...\n");
-    fflush(stdout);
-
-    memset(chunk, 0, CHUNK_SIZE);
-
-    /* private blocks have bit maps so mark off 32 bits for the bit map itself */
-    chunk[0] = 0xff;
-    chunk[1] = 0xff;
-    chunk[2] = 0xff;
-    chunk[3] = 0xff;
-
-    blocks_written = write_chunks(obj_fd, chunk, reserved_private_blocks / BLOCKS_IN_CHUNK);
-
-    if (blocks_written < reserved_private_blocks)
+    if (is_file)
     {
-	fprintf(stderr, "Error: not enough space for private chunks\n");
-	close(obj_fd);
-	exit(1);
+	printf("Writing private chunks...\n");
+	fflush(stdout);
+
+	memset(chunk, 0, CHUNK_SIZE);
+
+	/* private blocks have bit maps so mark off 32 bits for the bit map itself */
+	chunk[0] = 0xff;
+	chunk[1] = 0xff;
+	chunk[2] = 0xff;
+	chunk[3] = 0xff;
+
+	blocks_written = write_chunks(obj_fd, chunk, reserved_private_blocks / BLOCKS_IN_CHUNK);
+
+	if (blocks_written < reserved_private_blocks)
+	{
+	    fprintf(stderr, "Error: not enough space for private chunks\n");
+	    close(obj_fd);
+	    exit(1);
+	}
+
+	blocks += blocks_written;
+	printf("blocks: %u  blocks_written: %u  reserved_private_blocks: %u\n", 
+	       blocks, blocks_written, reserved_private_blocks);
+
+	assert(blocks == reserved_private_blocks + BLOCKS_IN_CHUNK);
     }
 
-    blocks += blocks_written;
-
-
-    chunks = (float)blocks / (float)BLOCKS_IN_CHUNK;
-
-    printf("Total chunks: %.1f\n", chunks);
-
-
     if (close(obj_fd) != 0)
     {
 	perror(DEFAULT_FILE);
 	exit(1);
     }
 
+    printf("Total chunks: %u\n", num_chunks);
+
+
     snprintf(msg, sizeof(msg), "   private blocks: %9u  used: %9u", reserved_private_blocks, blocks);
     nwos_log(msg);
 
@@ -720,6 +759,8 @@ int main(int argc, char* argv[])
 
 	nwos_initialize_objectify(bf_key, linear, serial, DEFAULT_TYPE_RW, DEFAULT_FILE);
 
+	nwos_allocate_new_chunk(nwos_ref_to_word(&private_root_obj_ref));
+
 	root_obj_ref.id[0] = 0;
 	root_obj_ref.id[1] = 0;
 	root_obj_ref.id[2] = 0;
