@@ -18,6 +18,9 @@
 --      Boston, MA 02111-1307, USA.
 --
 -- $Log: reference_list.c,v $
+-- Revision 1.16  2007/06/30 14:14:01  jsedwards
+-- Added #ifdef around logging of add_to_reference_list, so it is off by default.
+--
 -- Revision 1.15  2007/05/28 16:03:25  jsedwards
 -- Added 'remove_from_references' and 'remove_from_reference_list' functions.
 --
@@ -624,11 +627,13 @@ void nwos_add_to_reference_list(ObjRef* ref, ObjRef* ref_list)
 
     assert(is_same_object(&cache->first_block->list.common_header.class_definition, &nwos_reference_list_class_ref));
 
+#ifdef LOG_REFERENCE_LIST
     snprintf(log_msg, sizeof(log_msg), "nwos_add_to_reference_list - object: %02x%02x%02x%02x - ref list: %02x%02x%02x%02x - num_refs: %d", 
 	    ref->id[0], ref->id[1], ref->id[2], ref->id[3],
 	    ref_list->id[0], ref_list->id[1], ref_list->id[2], ref_list->id[3],
 	    cache->num_refs);
     nwos_log(log_msg);
+#endif
 
     if (cache->num_refs <= MAX_REFS_IN_REF_LIST)    /* there is just one block */
     {
